var db = null;
// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js
angular.module('starter', ['ionic', 'starter.controllers', 'ngCordova'])

.run(function($ionicPlatform, $cordovaStatusbar, $cordovaNetwork, $cordovaGoogleAnalytics) {
  $ionicPlatform.ready(function() {
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if(window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }

    if(window.StatusBar) {
      // org.apache.cordova.statusbar required
      StatusBar.styleDefault();
      $cordovaStatusbar.style(1);
      $cordovaStatusbar.overlaysWebView(true);
    }

    window.network=$cordovaNetwork;
    try {
      var connectiontype = $cordovaNetwork.getNetwork(); // 'wifi','4G','3G','Edge' ...
      var isOnline = $cordovaNetwork.isOnline(); // Boolean
      var isOffline = $cordovaNetwork.isOffline(); // Boolean
      console.log('=== LouvreApp : ConnectionType : ', connectionType, ' - IsOnline : ', isOnline, ' - IsOffline : ', isOffline);
    } catch (e) {
      console.log('=== LouvreApp : Error in retrieving NetworkStatus :', JSON.stringify(e));
    };

    //GA
    if (typeof analytics !== "undefined") {
	    analytics.startTrackerWithId(localeConfig.GATrackerId);
		analytics.debugMode();
        console.log('=== LouvreApp : GA is here ! >', analytics);
    } else {
        console.log('=== LouvreApp : Google Analytics Unavailable');
    }

  });
})

.config(function($stateProvider, $urlRouterProvider) {
  $stateProvider

    .state('app', {
      url: "/app",
      abstract: true,
      templateUrl: "templates/menu.html",
      controller: 'AppCtrl'
    })

    .state('app.lists', {
      url: "/lists",
      views: {
        'menuContent' :{
          templateUrl: "templates/lists.html",
          resolve : {
            getGA : function($q,$window){
              var deferred = $q.defer();
            }
          },
          controller: 'ListsCtrl'
        }
      }
    })

    .state('app.home', {
      url: "/home",
      views: {
        'menuContent' :{
          templateUrl: "templates/home.html",
          controller: 'HomeCtrl'
        }
      }
    })

    .state('app.list', {
      url: "/lists/:listId",
      views: {
        'menuContent' :{
          templateUrl: "templates/list.html",
          controller: 'ListCtrl'
        }
      }
    })

  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/app/home');
});


var appLouvre = angular.module('starter.controllers', []);

//----------------------------------------------
// File : "js/controllers/main.controller.js"
//----------------------------------------------
appLouvre.controller('AppCtrl', function($scope, $ionicModal, $timeout, $rootScope, $window, $ionicSideMenuDelegate, $ionicScrollDelegate, $ionicSlideBoxDelegate, $cordovaMedia, $ionicLoading,$q,$interval){
  $rootScope.global = {search:''};
  $rootScope.lists =  null;
  $rootScope.masterpieceDetails = [];

  //GA tracking variables
  $rootScope.initTimerDone = false;
  $rootScope.maxNbItemsBuffered = 500;
  $rootScope.timerForPurgeQueueIfEmpty = 300000;
  $rootScope.timerForPurgeQueueIfNotEmpty = 5000;
  $rootScope.LivePrefix = "Live_";
  $rootScope.PostPrefix = "Post_";

  window.louvresApp={};
  window.louvresApp.device = ionic.Platform.device();
  window.louvresApp.platform = ionic.Platform.platform();
  window.louvresApp.isAndroid = ionic.Platform.isAndroid();
  window.louvresApp.isIOS = ionic.Platform.isIOS();
  window.louvresApp.isIPad = ionic.Platform.isIPad();
  window.louvresApp.isWebView = ionic.Platform.isWebView();
  window.louvresApp.isWindowsPhone = ionic.Platform.isWindowsPhone();
  window.louvresApp.version = ionic.Platform.version();

  $rootScope.louvresApp = $window.louvresApp;
  $rootScope.louvresApp.currentMp3=0;
  $rootScope.louvresApp.currentMp3Title='';
  $rootScope.louvresApp.currentMp3Index=null;
  $rootScope.louvresApp.media=null;
  $rootScope.louvresApp.mediaTimer=null;
  $rootScope.louvresApp.mediaStatus = 0;
  $rootScope.louvresApp.currentGuide = null;

  $scope.tourPlanCurrent = 0;
  $rootScope.louvresApp.visitPlan = '';
  $rootScope.tourTitle = '';
  $rootScope.tourByTimes = '';
  $rootScope.tourByTopics = '';

  $rootScope.disableAutoPlay=false;
  $rootScope.indexAudio=0;
  $rootScope.positionAudio=0;
  $rootScope.widthPlayerAudio='0%';

  $rootScope.datas=null;
  $rootScope.config=null;
  $scope.audioModalDatas=null;
  $scope.audioModal=null;

  $scope.masterpieceModal = null;

  $scope.forceTipsScreenDisplay = function(){
	window.localStorage['TipsAlreadyDisplayed'] = "";
	$scope.callGATrackPageView('HelpScreen');
  }

  $scope.isDeviceOnline = function(){
    if (window.network) {
    //console.log('=== LouvreApp : ConnectionType : ', window.network.getNetwork(), ' - IsOnline : ', window.network.isOnline(), ' - IsOffline : ', window.network.isOffline());
    return window.network.isOnline();
    } else {
      //console.log('=== LouvreApp : Network totally disabled');
    return false;
    };
  };

  $rootScope.preLoadMasterpiecesConfigFiles = function() {
	var nbMasterpieces = globalConfig.lists.length;
	for (var i = 0; i < nbMasterpieces; i++) {
		var masterpiece = globalConfig.lists[i];
		if ($rootScope.masterpieceDetails[masterpiece.id] == null) {
			//console.log("=== LouvreApp : Start of preLoadMasterpiecesConfigFiles  N° " + i + " : [" + globalConfig.lists[i].datasPath + "]");
			require([globalConfig.lists[i].datasPath + 'config.js'], function() {
					$rootScope.masterpieceDetails[masterpiece.id] = configDatas;
					if (i < nbMasterpieces)
						window.setTimeout(function() {$rootScope.preLoadMasterpiecesConfigFiles();}, 100);
				});
			return;
		}
	}
	//console.log("=== LouvreApp : All masterpieces preloaded !");
  }

  //param0 : 1 => PageView, 2 => Event
  //param1 : PageTitle if param0 = 1 or Category if param0 = 2
  //param2 : action  - param3 : label - param4 : value
  $scope.insertIntoQueue = function(param0, param1, param2, param3, param4) {
    var nbObjectsInQueue = 0;
    var temp = window.localStorage['NbObjectInQueue'];
      if (temp) nbObjectsInQueue = parseInt(temp);
      if (nbObjectsInQueue >= $rootScope.maxNbItemsBuffered) return;
    var callToBufferize = param0 + "|" + param1;
      if (param2) callToBufferize += "|" + param2;
      if (param3) callToBufferize += "|" + param3;
      if (param4) callToBufferize += "|" + param4;

    window.localStorage['CallBufferized_' + nbObjectsInQueue] = callToBufferize;
    window.localStorage['NbObjectInQueue'] = nbObjectsInQueue + 1;
  };

  $rootScope.purgeQueue = function() {
      //console.log("=== LouvreApp : PurgeQueue in progress ...");
      var nbObjectsInQueue = 0;
      var temp = window.localStorage['NbObjectInQueue'];

      if (temp) nbObjectsInQueue = parseInt(temp);
      if (nbObjectsInQueue == 0) {
        window.setTimeout(function() {$rootScope.purgeQueue();}, $rootScope.timerForPurgeQueueIfEmpty);
        //console.log("=== LouvreApp : No object to purge => Exit PurgeQueue");
        return;
      }
      if (typeof analytics !== "undefined" && $scope.isDeviceOnline()) {
        window.localStorage['NbObjectInQueue'] = nbObjectsInQueue - 1;
        var call = window.localStorage['CallBufferized_' + (nbObjectsInQueue - 1)];
        //console.log("=== LouvreApp : Call [" + call + "]");
        if (call) {
          var params = call.split('|');
          if (params[0] == "1")
            $scope.callGATrackLivePostPageView(false, params[1]);
          else if (params[0] == "2") {
            if (params.length == 3)
              $scope.callGATrackLivePostEvent(false, params[1], params[2]);
            else if (params.length == 4)
              $scope.callGATrackLivePostEvent(false, params[1], params[2], params[3]);
            else if (params.length == 5)
              $scope.callGATrackLivePostEvent(false, params[1], params[2], params[3], params[4]);
          }
        }
        var sleepTime = $rootScope.timerForPurgeQueueIfNotEmpty - ((Math.random() - 0.5) * ($rootScope.timerForPurgeQueueIfNotEmpty *0.6 * 2));
        //console.log("=== LouvreApp : Queue not empty, quick nap of " + sleepTime + " ms");
        window.setTimeout(function() {$rootScope.purgeQueue();}, sleepTime);
      }else {
        window.setTimeout(function() {$rootScope.purgeQueue();}, $rootScope.timerForPurgeQueueIfEmpty);
        //console.log("=== LouvreApp : " + nbObjectsInQueue + " objects in Queue but no Analytics or Online => Exit PurgeQueue");
      }
  };

  $scope.callGATrackPageView = function(screenTitle) {
    $scope.callGATrackLivePostPageView(true, screenTitle);
  }

  $scope.callGATrackEvent = function(category, action, label, value) {
    $scope.callGATrackLivePostEvent(true, category, action, label, value);
  }

  $scope.callGATrackLivePostPageView = function(isLiveCall, screenTitle) {
    var track = $rootScope.PostPrefix + screenTitle;
  if (isLiveCall)
    track = $rootScope.LivePrefix + screenTitle;
    //console.log('=== LouvreApp : CallGATrackView [' + screenTitle + ']');
    if (typeof analytics !== "undefined" && $scope.isDeviceOnline()) {
    analytics.trackView(track);
    //console.log('=== LouvreApp : CallGATrackView done !');
    } else {
    //console.log('=== LouvreApp : Google Analytics Unavailable');
    $scope.insertIntoQueue("1", screenTitle);
    };
  };

  $scope.callGATrackLivePostEvent = function(isLiveCall, category, action, label, value) {
  var track = $rootScope.PostPrefix + category;
  if (isLiveCall)
    track = $rootScope.LivePrefix + category;
    //console.log('=== LouvreApp : CallGATrackEvent [' + category + '] - [' + action + '] - [' + label + '] - [' + value + ']');
    if (typeof analytics !== "undefined" && $scope.isDeviceOnline()) {
    analytics.trackEvent(track, action, label, value);
    //console.log('=== LouvreApp : CallGATrackEvent done !');
    } else {
    //console.log('=== LouvreApp : Google Analytics Unavailable');
    $scope.insertIntoQueue("2", category, action, label, value);
    };
  };

  // 1=> Like, 2 => Dislike,
  $scope.storeLikeInfo = function(masterpieceId, guideId, likeDislike) {
  window.localStorage['LikeInformations-' + masterpieceId + '-' + guideId] = likeDislike;
  }

  $scope.retrieveLikeInfo = function(masterpieceId, guideId) {
  return window.localStorage['LikeInformations-' + masterpieceId + '-' + guideId];
  }

  $scope.dislike = function(masterpieceId, guideId){
	$rootScope.classNameLike = 'active-dislike';
	$scope.storeLikeInfo(masterpieceId, guideId, "2");
	$scope.callGATrackEvent("Feedback", "Dislike", masterpieceId);
	//console.log('=== LouvreApp : Dislike masterpiece ', masterpieceId, ' - GuideId ', guideId);
  };

  $scope.dislikeCurrent = function() {
	$scope.dislike($rootScope.datas.id, $rootScope.louvresApp.currentGuide.id);
  };

  $scope.like = function(masterpieceId, guideId) {
	$rootScope.classNameLike = 'active-like';
	$scope.storeLikeInfo(masterpieceId, guideId, "1");
	$scope.callGATrackEvent("Feedback", "Like", masterpieceId);
	//console.log('=== LouvreApp : Like masterpiece ', masterpieceId, ' - GuideId ', guideId);
  };

  $scope.likeCurrent = function() {
	$scope.like($rootScope.datas.id, $rootScope.louvresApp.currentGuide.id);
  };

  // Create the audio modal that we will use later
  $ionicModal.fromTemplateUrl('templates/audio.html', {
    scope: $scope,
    animation: 'slide-in-up',
    focusFirstInput:false
  }).then(function(modal) {
    $scope.audioModal = modal;
  });

  // Triggered in the audio modal to close it
  $scope.closeAudioMenu = function() {
    $scope.callGATrackEvent('Audio', 'Close_Menu', $rootScope.datas.id);
    $scope.audioModal.hide();
  };

  // Open the audio modal
  $scope.audioMenu = function() {
    $scope.audioModalDatas=$rootScope.config;
    // console.log('=== LouvreApp : audioMenu : $rootScope.config :',JSON.stringify($rootScope.config));
    $scope.callGATrackPageView('AudioMenu_' + $rootScope.datas.id);
    $scope.callGATrackEvent('Audio', 'Open_Menu', $rootScope.datas.id);
    $scope.audioModal.show();
  };


  // Create the visit types modal that we will use later
  $ionicModal.fromTemplateUrl('templates/visitTypes.html', {
    scope: $scope,
    animation: 'slide-in-up',
    focusFirstInput:false
  }).then(function(modal) {
    $scope.modal = modal;
  });

  $scope.closeVisitTypes = function(skipCallGATrackEvent) {
    if (!skipCallGATrackEvent)
      $scope.callGATrackEvent('TourPlan', 'Close');
    $scope.modal.hide();
  };

  // Open the modal
  $scope.visitTypes = function() {
    $scope.callGATrackPageView('TourPlanSelection');
    $scope.callGATrackEvent('TourPlan', 'Open');
	$rootScope.tourTitle = toursConfig.TourTitle;
	$rootScope.tourByTimes = toursConfig.TimeLists;
	$rootScope.tourByTopics = toursConfig.TopicLists;
    $scope.modal.show();
  };

  // Perform the visit types action when the user submits his choice
  $scope.doVisitTypes = function(typeTour, tourPlan) {
    var defer = $q.defer();
    $scope.tourPlanCurrent=tourPlan;
    var filteredList = null;

    defer.promise
    .then(function () {
		var tour = '';
		if (typeTour == 1)
			tour = _.find(toursConfig.TimeLists, {'id': tourPlan});
		else if (typeTour == 2)
			tour = _.find(toursConfig.TopicLists, {'id': tourPlan});
		if (tour == null || tour.Detail.length == 0) {
			$rootScope.lists = globalConfig.lists;
			$rootScope.louvresApp.visitPlan = toursConfig.TimeLists[0].ShortName;
		}
		else {
			filteredList = tour.Detail;
			$rootScope.louvresApp.visitPlan = tour.ShortName;
		}
    })
    .then(function () {
        if (filteredList) {
          $rootScope.lists = [];
          var newListFiltered= [];
          filteredList.forEach(function(entry) {
            newListFiltered.push(_.find(globalConfig.lists, {'id': entry}));
          });
          $rootScope.lists=newListFiltered;
          $ionicScrollDelegate.scrollTop();
        }
    })
    .then(function () {
      $scope.callGATrackEvent('TourPlan', 'Choose_Tour_Plan', tourPlan);
      $scope.closeVisitTypes(true);
    });

    defer.resolve();
  };

  $scope.toggleRightSideMenu = function() {
    $ionicSideMenuDelegate.toggleRight();
  };

  $scope.toggleLeftSideMenu = function() {
    $ionicSideMenuDelegate.toggleLeft();
  };

  $rootScope.startVisit = function(){
  $scope.callGATrackEvent("Browse", "Start_Visit");
  };

  $rootScope.backToList = function(){
  $scope.callGATrackEvent("Browse", "Back_To_List");
  };

  $rootScope.selectMasterpiece = function(masterpieceId, openMasterpieceModal){
  	$scope.callGATrackEvent("Browse", "Select_Masterpiece", masterpieceId);
  	if(openMasterpieceModal) {
  		openMasterpiece(masterpieceId);
  	}
  };

  $rootScope.closeMasterpiece = function(){
    closeMasterpiece();
  };

///---------------- AUDIO ------------------
  //::::::::::::::::::::::::::::::::::::
  // AUDIO : SELECT THE CURRENT ITEM MENU
  //...................................
  $rootScope.itemAudioClicked = function ($index) {
    // console.log('=== LouvreApp : itemAudioClicked : In ItemAudioClicked [', $index, ']');
    $rootScope.selectedAudioIndex = $index;
  };


  $rootScope.clearAllStatusAudio=function(){
  //console.log('=== LouvreApp : clearAllStatusAudio : In clearAllStatusAudio');
      if($rootScope.louvresApp.mediaTimer){
        //clearInterval($rootScope.louvresApp.mediaTimer);
		$interval.cancel($rootScope.louvresApp.mediaTimer);
        $rootScope.louvresApp.mediaTimer = null;
      }
      $rootScope.indexAudio=0;
      $rootScope.positionAudio=0;
      $rootScope.selectedAudioIndex=-1;
      $rootScope.louvresApp.currentMp3=0;
      $rootScope.louvresApp.currentMp3Title='';
      $rootScope.louvresApp.currentMp3Index=null;
  }


  //:::::::::::::::::::::::::::::::
  // AUDIO : STOP
  //...............................
  $rootScope.stopAudio = function(disableAutoPlay){
    //console.log('=== LouvreApp : stopAudio : init');
    $rootScope.disableAutoPlay=disableAutoPlay;
    if ($rootScope.louvresApp.media) {
      $rootScope.louvresApp.media.stop();
      $rootScope.louvresApp.mediaStatus = Media.MEDIA_STOPPED;
      $rootScope.louvresApp.media.release();
      $rootScope.louvresApp.media = null;
      $scope.callGATrackEvent('Audio', 'Stop', $rootScope.datas.id + '_' + ($rootScope.indexAudio + 1));
    }
    $rootScope.clearAllStatusAudio();
  };


  //:::::::::::::::::::::::::::::::
  // AUDIO : NEXT
  //...............................
  $rootScope.nextAudio = function() {
    //console.log('=== LouvreApp : nextAudio : init');
    if ($rootScope.indexAudio < $rootScope.config.audio.length - 1) {
      //console.log('=== LouvreApp : nextAudio : move from ' + $rootScope.indexAudio + ' to ' + ($rootScope.indexAudio + 1));
      $rootScope.playAudio($rootScope.indexAudio + 1, true, true);
      $scope.callGATrackEvent('Audio', 'Next', $rootScope.datas.id + '_' + ($rootScope.indexAudio + 1));
    }
  };

  //:::::::::::::::::::::::::::::::
  // AUDIO : PREVIOUS
  //...............................
  $rootScope.previousAudio = function(){
    //console.log('=== LouvreApp : previousAudio : In PreviousAudio');
    if ($rootScope.indexAudio > 0)
    {
      //console.log('=== LouvreApp : previousAudio :  PreviousAudio move from ' + $rootScope.indexAudio + ' to ' + ($rootScope.indexAudio - 1));
      $rootScope.playAudio($rootScope.indexAudio - 1, true, true);
      $scope.callGATrackEvent('Audio', 'Prev', $rootScope.datas.id + '_' + ($rootScope.indexAudio + 1));
    }
  };

  //:::::::::::::::::::::::
  // AUDIO : PLAY OR Resume
  //.......................
  $rootScope.playResumeAudio = function() {
    //console.log('=== LouvreApp : playResumeAudio : In PlayResumeAudio : ' + $rootScope.louvresApp.mediaStatus);
    if ($rootScope.louvresApp.media) {
        if ($rootScope.louvresApp.mediaStatus == Media.MEDIA_RUNNING || $rootScope.louvresApp.mediaStatus == Media.MEDIA_STARTING) {
          $rootScope.disableAutoPlay = true;
          $rootScope.louvresApp.media.pause();
          $rootScope.louvresApp.currentMp3Title = '暂停，点击继续欣赏';
          $rootScope.louvresApp.mediaStatus = Media.MEDIA_PAUSED;
          $scope.callGATrackEvent('Audio', 'Pause', $rootScope.datas.id + '_' + ($rootScope.indexAudio + 1));
        } else if ($rootScope.louvresApp.mediaStatus == Media.MEDIA_PAUSED || $rootScope.louvresApp.mediaStatus == Media.MEDIA_STOPPED) {
          $rootScope.disableAutoPlay = false;
          $rootScope.louvresApp.media.play();
          $rootScope.louvresApp.currentMp3Title = $rootScope.config.audio[$rootScope.indexAudio].title;
          $rootScope.louvresApp.mediaStatus = Media.MEDIA_RUNNING;
          $scope.callGATrackEvent('Audio', 'Restart', $rootScope.datas.id + '_' + ($rootScope.indexAudio + 1));
        }
    }
  }

  $rootScope.InitAndLaunchMedia = function(src) {
  	try {
  	  $rootScope.louvresApp.media = new Media(src, onSuccessPlay, onErrorPlay, mediaStatusCallback);
  	  $rootScope.louvresApp.mediaStatus = Media.RUNNING;
  	  $scope.callGATrackEvent('Audio', 'Play', $rootScope.datas.id + '_' + ($rootScope.indexAudio + 1));
      // LOAD TIMER
      if ($rootScope.louvresApp.mediaTimer == null) {
        $rootScope.louvresApp.mediaTimer = $interval(checkAudioDuration, 1000);
      }
  	  return $rootScope.louvresApp.media.play();
  	} catch (err) {
  	  console.error('=== LouvreApp : ERROR : playAudio : Error during launch audio ... maybe no audio device ---- NO AUDIO');
  	}
  }


  //::::::::::::::::::::::::::::::::::::
  // AUDIO : PLAY (or Pause if playing)
  //....................................
  $rootScope.playAudio = function(index, disableAutoPlay, fastLaunch){
    //console.log('=== LouvreApp : playAudio : In PlayAudio : ', index, ' - ', disableAutoPlay);
    //console.log('=== LouvreApp : playAudio : SrcBuild = [' + $rootScope.datas.pathMp3 + $rootScope.config.audio[index].mp3 + ' - TitleBuild : [' + $rootScope.config.audio[index].title + ']');
    var src = $rootScope.datas.pathMp3 + $rootScope.config.audio[index].mp3;
    var title = $rootScope.config.audio[index].title;

    //console.log('=== LouvreApp : playAudio : SrcBuild = [' + src + ' - rootScope : [' + $rootScope + ']');
    // Play a new song
    if($rootScope.louvresApp.currentMp3 !== src){
      $rootScope.disableAutoPlay=disableAutoPlay;
      $rootScope.stopAudio(disableAutoPlay);
      $rootScope.louvresApp.currentMp3=src;
      $rootScope.louvresApp.currentMp3Title=title;

      $rootScope.selectedAudioIndex=index;
      $rootScope.indexAudio = index;
      $rootScope.louvresApp.currentMp3Index=index;

  	  if (fastLaunch){
        $rootScope.InitAndLaunchMedia(src);
  	  }else{
  		  window.setTimeout(function() {$rootScope.InitAndLaunchMedia(src);}, 500);
      }
    }
  }

  //:::::::::::::::::::::::::::::::
  // AUDIO : CALLBACK PLAYER
  //...............................
  var onSuccessPlay = function(){
      //console.log("=== LouvreApp : onSuccessPlay : playAudio() : clip audio lu avec succès");
  }

  var onErrorPlay = function(err){
      //console.error("=== LouvreApp : onErrorPlay : playAudio() : erreur lors de la lecture du clip audio : " + JSON.stringify(err));
  }


  //:::::::::::::::::::::::::::::::
  // AUDIO : CALLBACK
  //...............................
  var mediaStatusCallback = function(status) {
    $rootScope.louvresApp.mediaStatus = status;
    if(status == Media.MEDIA_STARTING || status == Media.MEDIA_RUNNING){
      $rootScope.disableAutoPlay = false;
      return ;
    }
    if(status == Media.MEDIA_STOPPED && !$rootScope.disableAutoPlay){
      //:::::::::::::::::::::::::::::::
      // AUDIO : ONLY AUTOPLAY CALLBACK
      //...............................
      //console.log('=== LouvreApp : mediaStatusCallback : Media.MEDIA_STOPPED');
      //console.log('=== LouvreApp : mediaStatusCallback : the Length of Song' , $rootScope.config.audio.length);
      var lengthAudio= $rootScope.indexAudio + 1;//Add +1 , index start 0
      // Play next audio
      if($rootScope.config.audio && $rootScope.config.audio.length > lengthAudio){
        $rootScope.indexAudio++;
        //console.log('=== LouvreApp : mediaStatusCallback : Play next audio with this index',$rootScope.indexAudio);
        return $rootScope.playAudio($rootScope.indexAudio, false);
      } else if ($rootScope.louvresApp.media){
        $rootScope.indexAudio = 0;
        return $rootScope.stopAudio(true);
      }
      return $rootScope.indexAudio = 0;
    }
  }


  //:::::::::::::::::::::::::::::::
  // AUDIO : DURATION CALLBACK
  //...............................
  var secondsToTime = function(secs) {
    var hours = Math.floor(secs / 3600);

    var divisor_for_minutes = secs % 3600;
    var minutes = Math.floor(divisor_for_minutes / 60);

    var divisor_for_seconds = divisor_for_minutes % 60;
    var seconds = Math.ceil(divisor_for_seconds);

    //Prefix with 0;
    if (hours < 10) {hours='0'+hours;}
    if (minutes < 10) {minutes='0'+minutes;}
    if (seconds < 10) {seconds='0'+seconds;}

    var obj = {
        "h": hours,
        "m": minutes,
        "s": seconds
    };
    return obj;
  };

  var checkAudioDuration = function(){
    if($rootScope.louvresApp.media && $cordovaMedia){
      // get my_media position
      var my_media=$rootScope.louvresApp.media;
      my_media.getCurrentPosition(
          // success callback
          function(position) {
              if (position > -1) {
                var durationTotal = my_media.getDuration();
                $rootScope.widthPlayerAudio = Math.round(position*100/durationTotal)+'%';
                //console.log($rootScope.widthPlayerAudio);
                var timeLive = secondsToTime(position);
                $rootScope.positionAudio=timeLive.h + " : " + timeLive.m + " : " + timeLive.s;
              }
          },
          // error callback
          function(e) {
              console.error("=== LouvreApp : ERROR : checkAudioDuration : getting pos=",JSON.stringify(e));
          }
      );
    }
  };

  // Create the detail masterpiece modal that we will use to display quick display of masterpiece
  $ionicModal.fromTemplateUrl('templates/detailMasterpiece.html', {
    scope: $scope,
    animation: 'slide-in-up',
    focusFirstInput:false
  }).then(function(modal) {
    $scope.masterpieceModal = modal;
  });


  var closeMasterpiece = function() {
  	if($scope.masterpieceModal) $scope.masterpieceModal.hide();
    //cordova.plugins.Keyboard.close(); // Hack to force close keyboard on IOS7 and IOS8 after event close
    $rootScope.backToList();
  }

  var openMasterpiece = function(masterpieceId) {
    $rootScope.backToList();
    var defer = $q.defer(),
        currentData = _.find(globalConfig.lists, { 'id': "" + masterpieceId}),
        displayAgainSame = $rootScope.datas && currentData.id == $rootScope.datas.id,
        startSlideBox=0;

    defer.promise
    .then(function(){

        $rootScope.global.search='';
        $rootScope.datas = currentData;

        $rootScope.config = $rootScope.masterpieceDetails[currentData.id];
        //console.log('=== LouvreApp : $rootScope.config : ',JSON.stringify($rootScope.config));
        //console.log('=== LouvreApp : $rootScope.datas : ',JSON.stringify($rootScope.datas));

        $rootScope.louvresApp.currentGuide = _.find(guidesConfig.lists, {'id': currentData.GuideId});
        window.configDatas = null;

        if ($rootScope.louvresApp.isAndroid) {
          $rootScope.datas.pathMp3 = '/android_asset/www/'+$rootScope.datas.datasPath;
        } else {
          $rootScope.datas.pathMp3 = $rootScope.datas.datasPath;
        }

        $scope.nextSlide = function() { $ionicSlideBoxDelegate.next(); }
        $scope.previousSlide = function() { $ionicSlideBoxDelegate.previous(); }
    })


    .then(function(){
      if (currentData.GuideId > 0 && currentData.id > 0) {
        var likeDislike = $scope.retrieveLikeInfo(currentData.id, currentData.GuideId);
        //console.log('=== LouvreApp retrieveLikeInfo ',  currentData.id, ' - ', currentData.GuideId, ' - res : ', likeDislike);
        switch (likeDislike) {
          case "1":
            $rootScope.classNameLike = 'active-like';
            break;
          case "2":
            $rootScope.classNameLike = 'active-dislike';
            break;
          default :
            $rootScope.classNameLike = '';
            break;
        }
      }

    })

    .then(function(){
	  $ionicSlideBoxDelegate.slide(0);
	  window.setTimeout(function() {
		if ($ionicSlideBoxDelegate.currentIndex() == 0)
			$ionicSlideBoxDelegate.next();
		}, 10000);
      if($scope.callGATrackPageView && !displayAgainSame) $scope.callGATrackPageView('DetailMasterpiece_' + currentData.id);
    })

    .then(function(){
      if(!displayAgainSame || (displayAgainSame && $rootScope.positionAudio==0) ){
        $rootScope.stopAudio(true);
        // Always start by playing the First song ...
        $rootScope.playAudio(0, true);
      }
    })

    .then(function(){
      $scope.masterpieceModal.show();
    });


    defer.resolve();
  };

});


//----------------------------------------------
// File : "js/controllers/home.ctrl.js"
//----------------------------------------------
appLouvre.controller('HomeCtrl', function($scope,$rootScope,$ionicSideMenuDelegate) {

  //Init localisation of the App
  $rootScope.HomeTitleH1 = localeConfig.HomeTitleH1;
  $rootScope.HomeTitleH2 = localeConfig.HomeTitleH2;
  $rootScope.HomeLabelButton = localeConfig.HomeLabelButton;

  $ionicSideMenuDelegate.canDragContent(false);
  //Call GA
  $scope.callGATrackPageView('Home');
  //Start Purge GA calls process ...
  if (!$rootScope.initTimerDone) {
    //console.log("=== LouvreApp : InitTimer in progress .... Done");
    window.setTimeout(function() {$rootScope.purgeQueue();}, 5000);
    window.setTimeout(function() {$rootScope.preLoadMasterpiecesConfigFiles();}, 100);
    $rootScope.initTimerDone = true;
  }
});

//----------------------------------------------
// File : "js/controllers/list.ctrl.js"
//----------------------------------------------
appLouvre.controller('ListCtrl', function($scope,$rootScope,$ionicSlideBoxDelegate, $q, $stateParams, $ionicModal,$window) {
  //console.log("=== LouvreApp : Start ListController : " + new Date().getTime());
  var currentData = _.find(globalConfig.lists, { 'id': $stateParams.listId});
  var displayAgainSame = $rootScope.datas && currentData.id == $rootScope.datas.id;
  $rootScope.global.search='';
  $rootScope.datas = currentData;

  $rootScope.config = $rootScope.masterpieceDetails[currentData.id];
  //console.log('=== LouvreApp : $rootScope.config : ',JSON.stringify($rootScope.config));
  //console.log('=== LouvreApp : $rootScope.datas : ',JSON.stringify($rootScope.datas));
  $rootScope.louvresApp.currentGuide = _.find(guidesConfig.lists, {'id': currentData.GuideId});
  //console.log('=== LouvreApp : $rootScope.louvresApp.currentGuide : ',JSON.stringify($rootScope.louvresApp.currentGuide));
  window.configDatas = null;

  if ($rootScope.louvresApp.isAndroid) {
    $rootScope.datas.pathMp3 = '/android_asset/www/'+$rootScope.datas.datasPath;
  } else {
    $rootScope.datas.pathMp3 = $rootScope.datas.datasPath;
  }
  // if there is a current audio and the change the new detail view, then we stop all audio
  $scope.nextSlide = function() { $ionicSlideBoxDelegate.next(); }
  $scope.previousSlide = function() { $ionicSlideBoxDelegate.previous(); }

    if (currentData.GuideId > 0 && currentData.id > 0) {
    var likeDislike = $scope.retrieveLikeInfo(currentData.id, currentData.GuideId);
    //console.log('=== LouvreApp retrieveLikeInfo ',  currentData.id, ' - ', currentData.GuideId, ' - res : ', likeDislike);
    switch (likeDislike) {
      case "1":
      $rootScope.classNameLike = 'active-like';
      break;
      case "2":
      $rootScope.classNameLike = 'active-dislike';
      break;
      default :
      $rootScope.classNameLike = '';
      break;
    }
    }

  $scope.callGATrackPageView('DetailMasterpiece_' + currentData.id);
  if (displayAgainSame) {
	//console.log("=== LouvreApp : End ListController : " + new Date().getTime());
    return ;
  }

  $rootScope.stopAudio(true);
  // Always start by playing the First song ...
  $rootScope.playAudio(0, true);

  //console.log("=== LouvreApp : End ListController : " + new Date().getTime());
});

//----------------------------------------------
// File : "js/controllers/lists.ctrl.js"
//----------------------------------------------
appLouvre.controller('ListsCtrl', function($scope,$rootScope,$cordovaStatusbar,$ionicScrollDelegate,$ionicModal,$q,$window) {
  //Test OPEN POPUP without storage
  //window.localStorage['TipsAlreadyDisplayed']="false";

  $scope.$watch('global.search',function(newValue,oldValue){
      $ionicScrollDelegate.scrollTop();
  });

  $rootScope.searchFocus= function(){
    cordova.plugins.Keyboard.show();
  };

  $rootScope.searchBlur= function(){
    cordova.plugins.Keyboard.close();
  };

  $rootScope.louvresApp.visitPlan = toursConfig.TimeLists[0].ShortName;
  $rootScope.config=null;
  if(!$rootScope.lists) $rootScope.lists = globalConfig.lists;
  $scope.callGATrackPageView('ListMasterpieces_' + $scope.tourPlanCurrent);

  //Display the TipScreen one time automatically at the first launch
  var tipsAlreadyDisplayed = window.localStorage['TipsAlreadyDisplayed'];
  if (tipsAlreadyDisplayed  == "true")return ;

  window.localStorage['TipsAlreadyDisplayed'] = "true";
  var defer = $q.defer();
    defer.promise
    .then(function () {
        // Create the overlay modal
        $ionicModal.fromTemplateUrl('templates/startoverlay.html', {
          scope: $scope,
          animation: 'slide-in-up',
          focusFirstInput:false
        }).then(function(modal) {
          $scope.overlayStart = modal;
        }).then(function(){
          $scope.overlayStart.show();
        });

        // Triggered in the overlay modal to close it
        $scope.openStartOverlay = function() {
          $scope.overlayStart.show();
        };

        // Triggered in the overlay modal to close it
        $scope.closeStartOverlay = function() {
          $scope.overlayStart.hide();
        };
    });
  defer.resolve();
});
